<?php

/**
 * @file
 * Contains \Drupal\s3_db_backup\Adapter\S3DatabaseAWSBackupAdapter
 */

namespace Drupal\s3_db_backup\Adapter;

use Drupal\file\Entity\File;
use Drupal\s3_db_backup\S3DatabaseBackup;
use Drupal\s3_db_backup\S3DatabaseBackupClientInterface;
use Aws\S3\Exception\S3Exception;

/**
 * Class S3DatabaseAWSBackupAdapter.
 */
class S3DatabaseAWSBackupAdapter implements S3DatabaseBackupAdapterInterface {

  /**
   * @var \Drupal\s3_db_backup\S3DatabaseBackupClientInterface
   */
  protected $backupDatabaseClient;

  /**
   * @var \Drupal\s3_db_backup\S3DatabaseBackup
   */
  protected $backupDatabase;

  /**
   * @param \Drupal\s3_db_backup\S3DatabaseBackupClientInterface $backupDatabaseClient
   * @param \Drupal\s3_db_backup\S3DatabaseBackup $backupDatabase
   */
  public function __construct(S3DatabaseBackupClientInterface $backupDatabaseClient, S3DatabaseBackup $backupDatabase) {
    $this->backupDatabaseClient = $backupDatabaseClient;
    $this->backupDatabase = $backupDatabase;
  }

  /**
   * Export routine.
   *
   * @inherit
   */
  public function export(): int {
    $status = FALSE;
    $config = $this->backupDatabase->getConfig();

    // Make sure S3 is enabled
    if ($config && $config->get('s3.status')) {
      $handler = $this->backupDatabaseClient->getFileHandler();
      $handler->setupFile($this->backupDatabaseClient->getSettings());

      // Get file
      $file = $handler->getFile();
      if ($file) {
        // Get database export
        $export = $this->backupDatabaseClient->dump();
        $export->start($file->getFileUri());

        // Instantiate an Amazon S3 client.
        $s3 = $this->backupDatabase->getS3Client();

        try {
          // Make sure we have a s3 client.
          if ($s3) {
            // Upload file to S3. The file size and type are determined by the SDK.
            $objectKey = $file->getFileName();
            $result = $s3->putObject([
              'Bucket' => $config->get('s3.bucket'),
              'Key' => $objectKey,
              'Body' => fopen($file->getFileUri(), 'r'),
            ]);

            // Check result
            if ($result && $objectUrl = $result['ObjectURL']) {
              // Create a temporary file entity.
              $entity = File::create([
                'uri' => $file->getFileUri(),
                'uid' => \Drupal::currentUser()->id(),
              ]);
              $entity->save();

              // Add export history entry.
              $eid = $this->backupDatabase->addHistoryEntry([
                'fid' => $entity->id(),
                'name' => $file->getFileName(),
                'uri' => $file->getFileUri(),
                's3_object_uri' => $objectUrl,
                's3_object_key' => $objectKey,
              ]);
              $status = $eid;
            }
          }
        } catch (S3Exception|\Exception $e) {
          \Drupal::logger('s3_db_backup')->error($e->getMessage());
        }
      }
    }

    return (int) $status;
  }

}
