<?php

/**
 * @file
 * Contains \Drupal\s3_db_backup\Controller\S3DatabaseBackupController.
 */

namespace Drupal\s3_db_backup\Controller;

use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Form\FormBuilder;
use Drupal\s3_db_backup\S3DatabaseBackup;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class S3DatabaseBackupController.
 */
class S3DatabaseBackupController extends ControllerBase {

  /**
   * @var \Drupal\s3_db_backup\S3DatabaseBackup
   */
  protected $databaseBackup;

  /**
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * @var \Drupal\Core\Form\FormBuilder
   */
  protected $formBuilder;

  /**
   * @param \Drupal\s3_db_backup\S3DatabaseBackup $databaseBackup
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   * @param \Drupal\Core\Form\FormBuilder $formBuilder
   */
  public function __construct(S3DatabaseBackup $databaseBackup, DateFormatter $dateFormatter, FormBuilder $formBuilder) {
    $this->databaseBackup = $databaseBackup;
    $this->dateFormatter = $dateFormatter;
    $this->formBuilder = $formBuilder;
  }

  /**
   * Inject services
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *
   * @return static
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('s3_db_backup'),
      $container->get('date.formatter'),
      $container->get('form_builder')
    );
  }

  /**
   * Backup history overview.
   *
   * @return array
   */
  public function getExportHistory(): array {
    $config = $this->databaseBackup->getConfig();

    // Build query
    $query = $this->databaseBackup->getConnection()->select('s3_db_backup', 'e');
    $count_query = clone $query;
    $count_query->addExpression('Count(e.eid)');

    $paged_query = $query->extend('Drupal\Core\Database\Query\PagerSelectExtender');

    $pagingLimit = 10;
    if ($config->get('page_limit')) {
      $pagingLimit = $config->get('page_limit');
    }
    $paged_query->limit($pagingLimit);
    $paged_query->setCountQuery($count_query);

    // Get export history
    $results = $paged_query
      ->fields('e', [
        'eid',
        'fid',
        'name',
        'uri',
        's3_object_uri',
        's3_object_key',
        'created',
      ])
      ->orderBy('created', 'DESC')
      ->execute()
      ->fetchAll();

    $rows = [];
    foreach ($results as $result) {
      // Setup data for table
      $url = $this->databaseBackup->getAdjustedUri($result->uri);
      $location = Link::fromTextAndUrl($this->t('Download'), Url::fromUri(file_create_url($url), ['attributes' => ['class' => 'button']]));
      $created = $this->dateFormatter->format($result->created, $config->get('date'));

      // Attempt to get pre-signed url
      $presignedUrl = NULL;
      if (!empty($result->s3_object_key)) {
        $presignedUrl = $this->databaseBackup->getPresignedUrl($result->s3_object_key);
      }

      // Setup row
      $rows[$result->fid] = [
        'eid' => $result->eid,
        'fid' => $result->fid,
        'name' => $result->name,
        'uri' => $location,
        's3_object_uri' => ($presignedUrl) ? Link::fromTextAndUrl($this->t('Download'), Url::fromUri($presignedUrl, ['attributes' => ['class' => 'button']])) : '',
        's3_object_key' => !empty($result->s3_object_key) ? $result->s3_object_key : '',
        'created' => $created,
      ];
    }

    // Return export history form.
    return [
      'form' => $this->formBuilder->getForm('Drupal\s3_db_backup\Form\S3DatabaseBackupHistoryForm', $rows),
      'pager' => [
        '#type' => 'pager',
        '#weight' => 5,
      ],
    ];
  }

}
