<?php

/**
 * @file
 * Contains \Drupal\s3_db_backup\Form\S3DatabaseBackupForm.
 */

namespace Drupal\s3_db_backup\Form;

use Drupal\s3_db_backup\S3DatabaseBackup;
use Drupal\s3_db_backup\S3DatabaseBackupClient;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;

use Drupal\s3_db_backup\Adapter\S3DatabaseLocalBackupAdapter;
use Drupal\s3_db_backup\Adapter\S3DatabaseRemoteBackupAdapter;
use Drupal\s3_db_backup\Adapter\S3DatabaseAWSBackupAdapter;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class S3DatabaseBackupForm.
 */
class S3DatabaseBackupForm extends FormBase {

  /**
   * @var \Drupal\s3_db_backup\S3DatabaseBackupClient
   */
  protected $backupDatabaseClient;

  /**
   * @var \Drupal\s3_db_backup\S3DatabaseBackup
   */
  protected $backupDatabase;

  /**
   * @param \Drupal\s3_db_backup\S3DatabaseBackupClient $backupDatabaseClient
   * @param \Drupal\s3_db_backup\S3DatabaseBackup $backupDatabase
   */
  public function __construct(S3DatabaseBackupClient $backupDatabaseClient, S3DatabaseBackup $backupDatabase) {
    $this->backupDatabaseClient = $backupDatabaseClient;
    $this->backupDatabase = $backupDatabase;
  }

  /**
   * Inject services
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *
   * @return static
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('s3_db_backup.client'),
      $container->get('s3_db_backup')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 's3_db_backup_export';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('s3_db_backup.settings');
    $site_name = \Drupal::config('system.site')->get('name');

    $form['backup'] = [
      '#type' => 'details',
      '#title' => $this->t('Database Backup'),
      '#open' => TRUE,
    ];

    $form['backup']['filename'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Filename'),
      '#description' => $this->t('The prefix name of the sql dump file.'),
      '#default_value' => $config->get('filename') ? $config->get('filename') : $site_name,
      '#field_suffix' => '.sql',
      '#size' => 40,
    ];
    if ($config->get('settings.compress') == 'Gzip') {
      $form['backup']['filename']['#field_suffix'] = '.sql.gz';
    }
    if ($config->get('settings.compress') == 'Bzip2') {
      $form['backup']['filename']['#field_suffix'] = '.sql.bz2';
    }

    $defaultExportType = 'local';
    $exportTypes = [
      'local' => $this->t('Local'),
      'download' => $this->t('Download'),
    ];
    if ($config->get('s3.status')) {
      $defaultExportType = 'aws';
      $exportTypes['aws'] = $this->t('AWS S3 Storage');
    }
    $form['backup']['type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Export type'),
      '#options' => $exportTypes,
      '#description' => $this->t('Export database backup to local, download or AWS S3.'),
      '#default_value' => $defaultExportType,
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Export Database'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    // Save filename.
    \Drupal::configFactory()->getEditable('s3_db_backup.settings')
      ->set('filename', $values['filename'])
      ->save();

    // Select our adapter.
    switch($values['type']) {
      case 'download':
        $handler = new S3DatabaseRemoteBackupAdapter($this->backupDatabaseClient, $this->backupDatabase);
        break;
      case 'aws':
        $handler = new S3DatabaseAWSBackupAdapter($this->backupDatabaseClient, $this->backupDatabase);
        break;
      default:
        $handler = new S3DatabaseLocalBackupAdapter($this->backupDatabaseClient, $this->backupDatabase);
        break;
    }

    // Run the export.
    if ($handler->export()) {
      $this->messenger()->addMessage($this->t('Database backup has been successfully completed.'), 'status');
      $options = [];
      $language = \Drupal::languageManager()->getCurrentLanguage();
      if (!empty($language)) {
        $options['language'] = $language;
      }
      $form_state->setRedirect('s3_db_backup.export_history', [], $options);
    }
    else {
      $this->messenger()->addMessage($this->t('Database backup has failed, please review recent log messages.'), 'warning');
    }
  }

}
