<?php

/**
 * @file
 * Contains \Drupal\s3_db_backup\Form\S3DatabaseBackupHistoryForm.
 */

namespace Drupal\s3_db_backup\Form;

use Drupal\s3_db_backup\S3DatabaseBackup;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Aws\S3\S3Client;
use Aws\S3\Exception\S3Exception;

/**
 * Class S3DatabaseBackupHistoryForm.
 */
class S3DatabaseBackupHistoryForm extends FormBase {

  /**
   * @var \Drupal\s3_db_backup\S3DatabaseBackup
   */
  protected $backupDatabase;

  /**
   * @param \Drupal\s3_db_backup\S3DatabaseBackup $backupDatabase
   */
  public function __construct(S3DatabaseBackup $backupDatabase) {
    $this->backupDatabase = $backupDatabase;
  }

  /**
   * Inject services
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *
   * @return static
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('s3_db_backup')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 's3_db_backup_history';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $rows = []): array {
    $header = [
      'eid' => $this->t('Export Id'),
      'fid' => $this->t('File Id'),
      'name' => $this->t('Export Filename'),
      'uri' => $this->t('Local Export File'),
      's3_object_uri' => $this->t('AWS S3 Export File'),
      's3_object_key' => $this->t('AWS S3 Object Key'),
      'created' => $this->t('Created'),
    ];

    $form['table'] = [
      '#type' => 'tableselect',
      '#header' => $header,
      '#options' => $rows,
      '#empty' => $this->t('No database export history found.'),
    ];

    if ($rows) {
      $form['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Delete Export(s)'),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->backupDatabase->getConfig();
    $values = $form_state->getValues();
    $files = array_filter($values['table']);
    foreach ($files as $fid) {
      $exportEntry = $this->backupDatabase->loadHistoryEntryByFileId($fid);
      if ($exportEntry) {
        try {
          if ($exportEntry->s3_object_key) {
            // Instantiate an Amazon S3 client.
            $s3 = new S3Client([
              'version' => 'latest',
              'region' => $config->get('s3.region'),
              'credentials' => [
                'key' => $config->get('s3.access_key'),
                'secret' => $config->get('s3.secret_key'),
              ],
            ]);
            // Make sure we have a s3 client.
            if ($s3) {
              // Delete object
              $result = $s3->deleteObject([
                'Bucket' => $config->get('s3.bucket'),
                'Key' => $exportEntry->s3_object_key,
              ]);
            }
          }
          // Remove history entry and file
          $this->backupDatabase->deleteHistoryEntry($fid);
        } catch (S3Exception|\Exception $e) {
          \Drupal::logger('s3_db_backup')->error($e->getMessage());
        }
      }
    }
    $this->messenger()->addMessage($this->t('Database backup(s) has been removed successfully.'));
  }

}
