<?php

namespace Drupal\s3_db_backup;

use Aws\S3\Exception\S3Exception;
use Aws\S3\S3Client;
use Drupal\Core\StreamWrapper\PrivateStream;
use Drupal\Core\File\FileSystem;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Connection;
use Drupal\Core\StreamWrapper\StreamWrapperManager;
use Drupal\file\Entity\File;

/**
 * Class S3DatabaseBackup.
 */
class S3DatabaseBackup {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * The config factory object.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * @var \Aws\S3\S3Client
   */
  protected $s3client;

  /**
   * @param \Drupal\Core\Database\Connection $connection
   * @param \Drupal\Core\File\FileSystem $filesystem
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   */
  public function __construct(Connection $connection, FileSystem $filesystem, ConfigFactory $config_factory) {
    $this->connection = $connection;
    $this->fileSystem = $filesystem;
    $this->configFactory = $config_factory;
  }

  /**
   * Get config.
   *
   * @return \Drupal\Core\Config\Config|\Drupal\Core\Config\ImmutableConfig
   */
  public function getConfig() {
    return $this->configFactory->get('s3_db_backup.settings');
  }

  /**
   * Get database connection.
   *
   * @return \Drupal\Core\Database\Connection
   */
  public function getConnection(): Connection {
    return $this->connection;
  }

  /**
   * Instantiate an Amazon S3 client.
   *
   * @return \Aws\S3\S3Client
   */
  public function getS3Client(): S3Client {
    if (!$this->s3client) {
      try {
        $config = $this->getConfig();
        // Instantiate an Amazon S3 client.
        $this->s3client = new S3Client([
          'version' => 'latest',
          'region' => $config->get('s3.region'),
          'credentials' => [
            'key' => $config->get('s3.access_key'),
            'secret' => $config->get('s3.secret_key'),
          ],
        ]);
      } catch (S3Exception|\Exception $e) {
        \Drupal::logger('s3_db_backup')->error($e->getMessage());
      }
    }
    return $this->s3client;
  }

  /**
   * Get presigned object url.
   *
   * @param string $objectKey
   * @param string $expires
   *
   * @return string|null
   */
  public function getPresignedUrl(string $objectKey, string $expires = '+60 minutes'): ?string {
    $presignedUrl = NULL;
    try {
      $config = $this->getConfig();
      $s3 = $this->getS3Client();
      $cmd = $s3->getCommand('GetObject', [
        'Bucket' => $config->get('s3.bucket'),
        'Key' => $objectKey,
      ]);
      $request = $s3->createPresignedRequest($cmd, $expires);
      $presignedUrl = (string) $request->getUri();
    } catch (S3Exception|\Exception $e) {
      \Drupal::logger('s3_db_backup')->error($e->getMessage());
    }
    return $presignedUrl;
  }

  /**
   * Get adjusted uri.
   *
   * file_create_url in combination with Url::fromUri does
   * not return a correct path for private files.
   *
   * @param string $uri
   *
   * @return string
   */
  public function getAdjustedUri(string $uri): string {
    $scheme = StreamWrapperManager::getScheme($uri);
    $path = $uri;
    if ($scheme == 'private') {
      $private_path = PrivateStream::basePath();
      $path = str_replace('private:/', $private_path, $uri);
    }
    return $path;
  }

  /**
   * Inserts a history entry.
   *
   * @param array $file
   *
   * @return \Drupal\Core\Database\StatementInterface|int|string|null
   * @throws \Exception
   */
  public function addHistoryEntry(array $file) {
    return $this->connection->insert('s3_db_backup')
      ->fields([
        'fid',
        'name',
        'uri',
        's3_object_uri',
        's3_object_key',
        'created',
      ])
      ->values([
        'fid' => $file['fid'],
        'name' => $file['name'],
        'uri' => $file['uri'],
        's3_object_uri' => !empty($file['s3_object_uri']) ? $file['s3_object_uri'] : NULL,
        's3_object_key' => !empty($file['s3_object_key']) ? $file['s3_object_key'] : NULL,
        'created' => \Drupal::time()->getRequestTime(),
      ])
      ->execute();
  }

  /**
   * Load history entry by primary id.
   *
   * @param int $id
   *
   * @return mixed
   */
  public function loadHistoryEntry(int $id) {
    return $this->connection->query('SELECT * FROM {s3_db_backup} WHERE eid = :id', [':id' => $id])
      ->fetchObject();
  }

  /**
   * Load history entry by file id.
   *
   * @param int $fid
   *
   * @return mixed
   */
  public function loadHistoryEntryByFileId(int $fid) {
    return $this->connection->query('SELECT * FROM {s3_db_backup} WHERE fid = :id', [':id' => $fid])
      ->fetchObject();
  }

  /**
   * Remove history entry by file id.
   *
   * @param int $fid
   *
   * @return void
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function deleteHistoryEntry(int $fid) {
    $this->connection->delete('s3_db_backup')
      ->condition('fid', $fid)
      ->execute();

    // Remove file and managed entry.
    $file = File::load($fid);
    if ($file) {
      $file->delete();
    }
  }

  /**
   * Show tables on current database.
   *
   * @return mixed
   */
  public function showTables() {
    return $this->connection->query('SHOW TABLES')->fetchAll();
  }

  /**
   * Format results as select list options.
   *
   * @param array $results
   *
   * @return array
   */
  public function formatOptions(array $results): array {
    $options = [];
    foreach ($results as $result) {
      $value = current((array) $result);
      $options[$value] = $value;
    }
    return $options;
  }

}
