<?php

/**
 * @file
 * Contains \Drupal\s3_db_backup\S3DatabaseBackupFileHandler
 */

namespace Drupal\s3_db_backup;

use Drupal\Core\File\FileSystem;
use Drupal\Core\File\FileSystemInterface;

/**
 * Class S3DatabaseBackupFileHandler.
 */
class S3DatabaseBackupFileHandler {

  /**
   * @var \Drupal\s3_db_backup\S3DatabaseBackupFile
   */
  protected $file;

  /**
   * @var \Drupal\Core\File\FileSystem
   */
  protected $filesystem;

  /**
   * File config
   *
   * @var array
   */
  protected $config;

  /**
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * @param \Drupal\s3_db_backup\S3DatabaseBackupFile
   *    The database export file to act on.
   *
   * @param \Drupal\Core\File\FileSystem $filesystem
   *    Provides helpers to operate on files and stream wrappers.
   */
  public function __construct(S3DatabaseBackupFile $file, FileSystem $filesystem) {
    $this->file = $file;
    $this->filesystem = $filesystem;
    $this->logger = \Drupal::logger('s3_db_backup');
  }

  /**
   * Handles file generation & setup.
   *
   * @param $config
   */
  public function setupFile($config) {
    $this->setConfig($config);

    $path = $this->createFilePath();
    $extension = $this->createFileType();

    $this->file->setFileName($this->config['name'] . '_' . time() . $extension);
    $this->file->setFileUri($this->filesystem->createFilename(
      $this->file->getFileName(), $path
    ));
  }

  /**
   * Get file.
   *
   * @return \Drupal\s3_db_backup\S3DatabaseBackupFile
   */
  public function getFile(): S3DatabaseBackupFile {
    return $this->file;
  }

  /**
   * Set file.
   *
   * @param \Drupal\s3_db_backup\S3DatabaseBackupFile $file
   *
   * @return void
   */
  public function setFile(S3DatabaseBackupFile $file) {
    $this->file = $file;
  }

  /**
   * Set default file info.
   *
   * @param $config
   */
  private function setConfig($config) {
    $dateFormatter = \Drupal::service('date.formatter');
    $this->config = [
      'date' => $dateFormatter->format(time(), $config->get('date')),
      'compress' => $config->get('settings.compress'),
      'name' => $config->get('filename'),
      'path' => $config->get('path'),
    ];
  }

  /**
   * Create export location.
   *
   * @return false|mixed|string
   */
  private function createFilePath() {
    $result = $this->config['path'];

    if (!$this->filesystem->prepareDirectory($result, FileSystemInterface::CREATE_DIRECTORY)) {
      $result = FALSE;
      $this->logger->error('The requested directory @dir could not be created.', ['@dir' => $this->config['path']]);
    }
    else {
      if (!$this->filesystem->prepareDirectory($result)) {
        $result = FALSE;
        $this->logger->error('The requested directory @dir permissions are not writable.', ['@dir' => $this->config['path']]);
      }
    }

    if ($this->config['date']) {
      $filepath = $this->config['path'] . '/' . $this->config['date'];
      if ($this->filesystem->prepareDirectory($filepath, FileSystemInterface::CREATE_DIRECTORY)) {
        $result = $filepath;
      }
    }

    return $result;
  }

  /**
   * Determine the export type & extension.
   *
   * @return string
   */
  private function createFileType(): string {
    $type = 'application/octet-stream';
    $extension = '.sql';

    switch ($this->config['compress']) {
      case 'Gzip':
        $extension .= '.gz';
        $type = 'application/gzip';
        break;
      case 'Bzip2':
        $extension .= '.bz2';
        $type = 'application/x-bzip2';
    }

    $this->file->setFileType($type);
    return $extension;
  }

}
